(ns ydn.core-test
  (:require [clojure.test :refer :all]
            [ydn.core :as sut]))

(deftest ref-kw-test
  (is (= ":ref.bar/baz"
         (sut/ref-kw ":ref.bar/baz"))))

(deftest key-to-path-test
  (is (= [:baz] (sut/key-to-path :ref/baz)))
  (is (= [:bar :baz] (sut/key-to-path :ref.bar/baz))))

(deftest apply-self-refs
  (is (= {:foo {:bar {:baz "32"}}
          :x {:bar {:baz "32"}}
          :y {:baz "32"}
          :z "32"}
         (sut/apply-self-refs
          {:foo {:bar {:baz "32"}}
           :x :ref/foo
           :y :ref.foo/bar
           :z :ref.foo.bar/baz})))

  (is (= {:pg {:domain "example.org", :port "8888", :scheme "https"},
          :i "https://example.org:8888"}
         (sut/apply-self-refs
          {:pg {:domain "example.org"
                :port "8888"
                :scheme "https"}
           :i "${:ref.pg/scheme}://${:ref.pg/domain}:${:ref.pg/port}"})))

  (is (= {:pg {:domain "example.org"
               :port "8888"
               :scheme "https"}
          :x :pg/domain,
          :i "${:pg/scheme}://${:pg/domain}:${:pg/port}"}
         (sut/apply-self-refs
          {:pg {:domain "example.org"
                :port "8888"
                :scheme "https"}
           :x :pg/domain
           :i "${:pg/scheme}://${:pg/domain}:${:pg/port}"}))
      "No interpolation or self-ref if the keyword has not ref for root-namespace"))

(deftest fix-apply-self-refs
  (is (= {:x 42
          :y 42
          :z 42
          :t 42
          :u 42}
         (sut/fix-apply-self-refs
          10
          {:x 42
           :y :ref/x
           :z :ref/y
           :t :ref/z
           :u :ref/t})))

  (is (= {:ex-message
          "Too many loops! (10) You probably have a dependency cycle in your config."
          :ex-data
          {:latest
           {:x 42
            :y
            "42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + 42 + ${:ref/x} + ${:ref/y}"
            :z "42 + z"}}}
         (try
           (sut/fix-apply-self-refs
            10
            {:x 42
             :y :ref/z
             :z "${:ref/x} + ${:ref/y}"})
           (catch Exception e
             {:ex-message (ex-message e)
              :ex-data (ex-data e)}))))

  (is (= {:ex-message
          "Too many loops! (3) You probably have a dependency cycle in your config.",
          :ex-data
          {:latest
           {:x 42,
            :z
            "${:ref/z} + ${:ref/z} + ${:ref/z} + ${:ref/z} + ${:ref/z} + ${:ref/z} + ${:ref/z} + ${:ref/z} + ${:ref/z} + ${:ref/z} + ${:ref/z} + ${:ref/z} + ${:ref/z} + ${:ref/z} + ${:ref/z} + ${:ref/z}"}}}
         (try
           (sut/fix-apply-self-refs
            3
            {:x 42
             :z "${:ref/z} + ${:ref/z}"})
           (catch Exception e
             {:ex-message (ex-message e)
              :ex-data (ex-data e)})))))
